Choice

Enum Choice 

pub enum Choice {
    DoNothing,
    Move(Input),
    Merge(Input),
    Drop(HashSet<TableId, FxBuildHasher>),
}
Describes what to do (compact or not)

Variants



DoNothing

Just do nothing.



Move(Input)

Moves tables into another level without rewriting.



Merge(Input)

Compacts some tables into a new level.



Drop(HashSet<TableId, FxBuildHasher>)

Delete tables without doing compaction.

This may be used by a compaction strategy that wants to delete old data without having to compact it away, like fifo::Strategy.