Memtable

Struct Memtable 

pub struct Memtable { /* private fields */ }
The memtable serves as an intermediary, ephemeral, sorted storage for new items

When the Memtable exceeds some size, it should be flushed to a table.

Implementations

Source

impl Memtable

pub fn id(&self) -> MemtableId

Returns the memtable ID.

pub fn is_flagged_for_rotation(&self) -> bool

Returns true if the memtable was already flagged for rotation.

pub fn flag_rotated(&self)

Flags the memtable as requested for rotation.

pub fn iter(&self) -> impl DoubleEndedIterator<Item = InternalValue> + '_

Creates an iterator over all items.

pub fn size(&self) -> u64

Gets approximate size of memtable in bytes.

pub fn len(&self) -> usize

Counts the number of items in the memtable.

pub fn is_empty(&self) -> bool

Returns true if the memtable has no KV items and no range tombstones.

pub fn insert_range_tombstone( &self, start: UserKey, end: UserKey, seqno: SeqNo, ) -> u64

Inserts a range tombstone covering [start, end) at the given seqno.

Returns the approximate size added to the memtable.

Returns 0 if start >= end or if either bound exceeds u16::MAX bytes.

Panics

Panics if the internal RwLock is poisoned.

pub fn range_tombstone_count(&self) -> usize

Returns the number of range tombstones.

Panics

Panics if the internal RwLock is poisoned.

pub fn get_highest_seqno(&self) -> Option<SeqNo>

Returns the highest sequence number in the memtable.